
/*
 * sentryc.c - sentry network client
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#include <signal.h>
#include <syslog.h>
#include <errno.h>

#include "sentry.h"
#include "sentryc.h"

int debug;
int verbose;

char    sentry_server[128];
int     ctrlfd = -1;
int     datafd = -1;
int     data_port;

#ifdef USE_LOCKFILE
char    lockfile[] = "/var/tmp/sentryc.pid";
#endif

int     new_udp_socket();
void    init_sentryc();
void    usage();

extern	void request_call();


void complete()
{
  extern int sentry_shmid;

  client_type = CLOSE;
  request_call();

#ifdef DELETE_IPC
  delete_pool(sentry_shmid);
#endif

  syslog(LOG_NOTICE, "sentryc closed.\n");
  closelog();

#ifdef USE_LOCKFILE
  unlink(lockfile);
#endif
  exit (0);
}


int new_udp_socket()
{
  int sockfd;
  int flag=1;

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("sentryc: cannot make new UDP socket");
    return -1;
  }
  setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&flag, sizeof(flag));

  return sockfd;
}

int build_ctrl_port(int port) 
{
  int sockfd;
  struct sockaddr_in server;
  struct hostent *hp;

  sockfd = new_udp_socket();

  bzero((char *)&server, sizeof(server));
  server.sin_family = AF_INET;
  server.sin_port = htons(server_port);
  if ((server.sin_addr.s_addr = inet_addr(sentry_server)) == INADDR_NONE) {
    hp = gethostbyname(sentry_server);
    if (hp->h_addrtype != AF_INET) {
      fprintf(stderr, "%s: invalid address type.\n", sentry_server);
      close(sockfd);
      return -1;
    }
    bcopy(hp->h_addr, (char *)&server.sin_addr.s_addr, hp->h_length);
  }

  if (connect(sockfd, (struct sockaddr *)&server, sizeof(server)) < 0) {
    perror("cannot bind or cannect");
    close(sockfd);
    return -1;
  }
  fprintf(stderr, "prepared ctrl socket# %d\n", sockfd);

  return sockfd;
}

int build_data_port(int port)
{
  int sockfd;
  struct sockaddr_in me;
  int namelen;
  
  sockfd = new_udp_socket();
  
  bzero((char *)&me, sizeof(me));
  me.sin_family = AF_INET;
  me.sin_addr.s_addr = htonl(INADDR_ANY);
  me.sin_port = htons(port);

  if (bind(sockfd, (struct sockaddr *)&me, sizeof(me)) < 0) {
    perror("Cannot bind data socket");
    close(sockfd);
    return -1;
  }

  /* get data port# */
  if (getsockname(sockfd, (struct sockaddr *)&me, &namelen) < 0) {
    perror("Cannot get data sockname");
    close(sockfd);
    return -1;
  }
  data_port = ntohs(me.sin_port);

  fprintf(stderr, "prepared data port# %d, ctrlfd# %d.\n", 
	  data_port, sockfd);

  return sockfd;
}


void
mainloop(void)
{
  extern u_char client_type;

  init_ibuf();

  init_request();
  request_call();
  signal(SIGALRM, request_call);

  while (1) {
    get_data(datafd);
  }
}

int
main(int argc, char **argv)
{
  int c;
  extern char *optarg;
  extern int optind;

#ifdef DEBUG
  debug = 1;
#else
  debug = 0;                            /* debug flag */
#endif
#ifdef VERBOSE
  verbose = 1;                         /* verbose flag */
#else
  verbose = 0;                          
#endif



  openlog("sentryc", LOG_PID, LOG_DAEMON);
  syslog(LOG_NOTICE, "sentryc starts.\n");

  signal(SIGQUIT, complete);
  signal(SIGINT, complete);
  signal(SIGTERM, complete);
  signal(SIGALRM, SIG_IGN);

  while ((c = getopt(argc, argv, "hdv:p:s")) != -1) {
    switch(c) {
    case 'h': /* help */
    default:
      usage();
      exit (1);
    case 'd':
      debug ++;
      break;
    case 'v': /* verbose mode */
      verbose = 1;
      break;
    case 'p':
      server_port = atoi(optarg);
      break;
    case 's':
      shm_size = atoi(optarg);
      break;
    }      
  }

  if (argc <= 1) 
    usage();

  /*
   * daemonize if debug == 0
   */
  if (debug == 0) { /* future work */ }

  strcpy(sentry_server, argv[argc-1]);

#if USE_LOCKFILE
  init_sentryc();
#endif

  ctrlfd = build_ctrl_port(0);
  datafd = build_data_port(7000);

  if (ctrlfd < 0 || datafd < 0) 
    exit (1);

  (void)mainloop();
}


void
init_sentryc()
{
#ifdef USE_LOCKFILE
  int chk;
  FILE *fp;
  char mesg[80];

  if (debug)
    fprintf(stderr, "init_sentryc: lockfile=%s\n", lockfile);

  while (1) {
    chk = access(lockfile, F_OK);
    if (chk < 0) break;
    
    if (errno == EINTR)
      continue;

    sprintf(mesg, "init_sentryc: locckfile(%s) is found", lockfile);
    perror(mesg);
    exit (1);
  }

  if ((fp = fopen(lockfile, "w")) == NULL) {
    sprintf(mesg, "init_sentryc: cannot open lockfile(%s)", lockfile);
    perror(mesg);
    exit (1);
  }

  sprintf(mesg, "%d\n", getpid());
  fprintf(stderr, "pid# %d\n", getpid());
  
  if (fputs(mesg, fp) == EOF) {
    perror("init_sentryc: cannot fputs()");
    exit (1);
  }
  fclose(fp);
#endif /* USE_LOCKFILE */
}
  

void
usage(void)
{
  fprintf(stderr, "Usage: sentryc [options] hostname.\n\n");
  fprintf(stderr, "%s%s%s%s%s%s%s%s%s%s\n",
	  "option:\n",
	  "        -h         help\n",
	  "        -t type    unicast(0)/multicast(1) [default: unicast(0)]\n",
	  "        -c color   monochrome(0)/color(1)  [default: color(1)]\n",
	  "        -s type    frame size\n",
	  "        \n",
	  "        \n",
	  "        \n",
	  "        \n",
	  "        -r num     frame rate (fps)        [default: 10 fps]\n");

  exit (1);
}
